from win32api import GetSystemMetrics
from comtypes.client import CreateObject
from comtypes.gen import STKObjects
from comtypes import COMError

class Scenario:

    def __init__(self, application, name, timePeriod):
        
        
        self.__guardian = application
        self.name = name.replace(' ','_')
        
        application.root.NewScenario(self.name)
        self.__scenario = application.root.CurrentScenario
        self.__scenario = self.__scenario.QueryInterface(STKObjects.IAgScenario)
        try:
            self.__scenario.SetTimePeriod('Today',str(timePeriod))
        except COMError:
            raise ValueError, "Time period not properly formatted"

    def SetTimePeriod(self, elapsedTime):

        self.__scenario.SetTimePeriod('Today',str(elapsedTime))
        
    def GetReference(self):
        return self.__scenario

    def Close(self):

        self.__guardian.root.CloseScenario()

    def GetGuardian(self):
        return self.__guardian




import os

class TLE_Manager:
    
    
    """
    
    Class containing only static methods for processing TLE files.
    
    """
    
    @staticmethod
    def ParseTLE(filename):
    
        """
        
        Parses a TLE file of the given file name.
        
        Parameters:
            filename (str): Name of the TLE file.
            
        """
    
        #Create array for return value
        outputArray = []
    
        #Open TLE file
        tle = open(filename,'r')
    
        #Saves first and second lines to variables
        firstLine = tle.readline()
        secondLine = tle.readline()
    
        #Adds line to output array
        outputArray.append(firstLine)
        outputArray.append(secondLine)
    
        #Close tle file
        tle.close()
    
        return outputArray

    @staticmethod
    def GenerateTLE(application, sscNumber):
    
        """
    
        TESTING REQUIRED

        Retrieves TLE file for a satellite with a given SSC Number

        Parameters:
            application (uiApplication): Application object that holds the scenario.
            sscNumber (str || int): SSC Number of the satellite to retrieve TLE file from.
    
        """
        
        sscNumber = str(sscNumber)
        application.root.ExecuteCommand("CreateTLEFile * AGIServer " +
                os.getcwd() + "\\" + sscNumber + ".tle" + 
                " SSCNumber " + sscNumber)
   
class Toolbox:

    @staticmethod
    def ComputeCenterTarget(parsedLine):
        
        """
        
        Computes the coordinates for the center of an area target.
        
        Parameters:
            parsedLine (str): Line parsed from target list csv.
            
        Returns:
            midPoint (list): List containing name and coordinates of the middle of the area target.
            
        """
        
        name = parsedLine[0]
        startLat = parsedLine[8]
        startLon = parsedLine[9]
        endLat = parsedLine[10]
        endLon = parsedLine[11]
        
        midLat = (float(startLat) + float(endLat))/2.0
        midLon = (float(startLon) + float(endLon))/2.0
        
        return (midLat,midLon)
    
    @staticmethod
    def GetTimeDelta(timeArray):
        
        """
        
        Computes the time difference between to time instances from STK.
        
        Parameters:
            timeArray (list): A list containting two time instances from STK.
            Each instance is in format: 'DD MM YYYY HH:mm:SS.sss'
            
        Returns:
            deltat (float): Time elapsed in seconds between the two instants.
            
        """
        
        monthDict = {"Jan": 1,
                     "Feb": 2,
                     "Mar": 3,
                     "Apr": 4,
                     "May": 5,
                     "Jun": 6,
                     "Jul": 7,
                     "Aug": 8,
                     "Sep": 9,
                     "Oct": 10,
                     "Nov": 11,
                     "Dec": 12}
        
        time1 = str(timeArray[0])
        
        splitTime1 = time1.split(' ')
        day1 = splitTime1[0]
        month1 = monthDict[splitTime1[1]]
        year1 = splitTime1[2]
        
        clockTime1 = splitTime1[3]
        
        splitClockTime1 = clockTime1.split(':')
        hours1 = splitClockTime1[0]
        minutes1 = splitClockTime1[1]
        seconds1 = splitClockTime1[2].split('.')[0]
        microseconds1 = splitClockTime1[2].split('.')[1]
        
        timeObj1 = datetime.datetime(
                int(year1),
                int(month1),
                int(day1),
                hour=int(hours1),
                minute=int(minutes1),
                second=int(seconds1),
                microsecond=int(microseconds1))
        
        time2 = str(timeArray[1])
        
        splitTime2 = time2.split(' ')
        day2 = splitTime2[0]
        month2 = monthDict[splitTime2[1]]
        year2 = splitTime2[2]
        
        clockTime2 = splitTime2[3]
        
        splitClockTime2 = clockTime2.split(':')
        hours2 = splitClockTime2[0]
        minutes2 = splitClockTime2[1]
        seconds2 = splitClockTime2[2].split('.')[0]
        microseconds2 = splitClockTime2[2].split('.')[1]
        
        timeObj2 = datetime.datetime(
                int(year2),
                int(month2),
                int(day2),
                hour=int(hours2),
                minute=int(minutes2),
                second=int(seconds2),
                microsecond=int(microseconds2))
        
        timeDiff = timeObj2 - timeObj1
        print(timeDiff.days,timeDiff.seconds, timeDiff.microseconds)
        deltat = float(timeDiff.days*86400.0) + \
        float(timeDiff.seconds) + \
        (0.001*float(timeDiff.microseconds))
                    
        return deltat
        



from win32api import GetSystemMetrics
from comtypes.client import CreateObject

class Application:
    def __init__(self):
        self.__uiApplication = CreateObject("STK11.Application")
        self.__uiApplication.Visible = True
        self.__uiApplication.UserControl = True
        
        self.root = self.__uiApplication.Personality2

    def Close(self):

        self.__uiApplication.Quit()




from win32api import GetSystemMetrics
from comtypes.client import CreateObject
from comtypes.gen import STKObjects
from comtypes import COMErrorpi

from toolbox import TLE_Manager, Toolbox

class AreaTarget:
    
    def __init__(self, scenario, parsedLine):
        self.__guardian = scenario
        
        root = self.__guardian.GetGuardian().root
        
        root.BeginUpdate()
        
        self.ID = parsedLine[1]
        startLat = parsedLine[8]
        startLon = parsedLine[9]
        endLat = parsedLine[10]
        endLon = parsedLine[11]
        
        self.center =Toolbox.ComputeCenterTarget(parsedLine)

        self.__areaTarget = root.CurrentScenario.Children.New(STKObjects.eAreaTarget,self.ID)
        self.__areaTarget = self.__areaTarget.QueryInterface(STKObjects.IAgAreaTarget)
        self.__areaTarget.AreaType = STKObjects.ePattern
        self.__patterns = self.__areaTarget.AreaTypeData
        self.__patterns = self.__patterns.QueryInterface(STKObjects.IAgAreaTypePatternCollection)
        
        self.__patterns.Add(startLat,startLon)
        self.__patterns.Add(startLat,endLon)
        self.__patterns.Add(endLat,endLon)
        self.__patterns.Add(endLat,startLon)
        self.__areaTarget.AutoCentroid = True
        
        root.EndUpdate()

    def GetTarget(self):

        return self.__areaTarget

    def GetPatterns(self):

        return self.__patterns

    def GetGuardian(self):

        return self.__guardian
    
class Camera:
    
    def __init__(self, hostSat, name, fov):
        self.__guardian = hostSat
        
        if not ((type(fov[0]) == int or type(fov[0]) == float) and
                (type(fov[1]) == int or type(fov[1]) == float)):
            raise TypeError, "Field of View parameters of invalid type."
        
        root = self.__guardian.GetGuardian().GetGuardian().root
        
        root.BeginUpdate()
        
        self.__cameraGen = self.__guardian.GetReference().Children.New(20,name)
        self.__camera = self.__cameraGen.QueryInterface(STKObjects.IAgSensor)
        self.__camera.CommonTasks.SetPatternRectangular(fov[0],fov[1])
        
        root.EndUpdate()
       
    def GetAccess(self, areaTarget):

        root = self.__guardian.GetGuardian().GetGuardian().root
        
        root.BeginUpdate()

        access = self.__cameraGen.GetAccessToObject(areaTarget.GetTarget())
        access.ComputeAccess()
        intervalCollection = access.ComputedAccessIntervalTimes
        try:
            computedIntervals = intervalCollection.ToArray(0,-1)
            root.EndUpdate()

            return computedIntervals
        except:
            root.EndUpdate()

            return 0
        
    def GetCamera(self):
        return self.__camera
    
    def GetCameraGen(self):
        return self.__cameraGen
    
    def GetGuardian(self):
        return self.__guardian

class Satellite:

    def __init__(self, scenario, name, sscNumber):
        self.__guardian = scenario
        self.name = name

        root = self.__guardian.GetGuardian().root

        TLE_Manager.GenerateTLE(self.__guardian.GetGuardian().root, str(sscNumber))
        self.tle = TLE_Manager.ParseTLE(str(sscNumber) + ".tle")
        
        try:
            self.__satellite = root.CurrentScenario.Children.New(STKObjects.eSatellite, name)
        except COMError:
            raise (RuntimeError,'\nIncorrect name format or name already taken for satellite.' + 
                  ' Please do not use spaces or reuse satellite names.')
            
        try:
            root.ExecuteCommand('SetState */Satellite/' + self.name + ' TLE "' +
                                     self.tle[0] + '" "' + self.tle[1] +
                                     '" TimePeriod "' +
                                     self.__guardian.GetReference().StartTime + '" "' +
                                     self.__guardian.GetReference().StopTime + '"')
        except COMError:
            raise (RuntimeError, "Failure to add satellite. Check formatting of TLE.")
    
    def ComputeKeplerians(self, timeInstant):
        
        satDPSingle = self.__satellite.DataProviders.Item('Classical Elements')
        satDPSingle = satDPSingle.QueryInterface(STKObjects.IAgDataProviderGroup)
        ceicrf = satDPSingle.Group.Item('ICRF')
        ceicrf = ceicrf.QueryInterface(STKObjects.IAgDataPrvTimeVar)

        result = ceicrf.ExecSingle(timeInstant)

        time = result.DataSets.GetDataSetByName('Time').GetValues()
        sma = result.DataSets.GetDataSetByName('Semi-major Axis').GetValues()
        ecc = result.DataSets.GetDataSetByName('Eccentricity').GetValues()
        inc = result.DataSets.GetDataSetByName('Inclination').GetValues()
        raan = result.DataSets.GetDataSetByName('RAAN').GetValues()
        aop = result.DataSets.GetDataSetByName('Arg of Perigee').GetValues()
        trueAnomaly = result.DataSets.GetDataSetByName('True Anomaly').GetValues()

        arr = [time,sma,ecc,inc,raan,aop,trueAnomaly]
        
        return arr

    def GetAccess(self, areaTarget):

        root = self.__guardian.GetGuardian().root

        root.BeginUpdate()

        access = self.__satellite.GetAccessToObject(areaTarget.GetTarget())
        access.ComputeAccess()

        intervalCollection = access.ComputedAccessIntervalTimes

        try:
            computedIntervals = intervalCollection.ToArray(0,-1)
            root.EndUpdate()

            return computedIntervals
        except Exception:
            root.EndUpdate()

            return 0
        
    def ComputeDSInfo(self, areaTarget, passArray):

        startTime = passArray[0]
        endTime = passArray[1]

        keplerians = self.ComputeKeplerians(startTime)

        return [areaTarget.ID, areaTarget.center, keplerians, (startTime, endTime)]


        keplerians = self.ComputeKeplerians(startTime)

        return [areaTarget.ID, areaTarget.center, keplerians, (startTime, endTime)]

    def GetReference(self):
        
        return self.__satellite
    
    def GetGuardian(self):
        return self.__guardian
